@@ -140,14 +140,10 @@ pub fn fetch_contest(
         Ok(contest_yaml) => parse_contest_yaml(&contest_yaml, problem_dir, template_dir),
         Err(_) => ContestResult {
             contest: None,
-            contest_err: Some(ContestErr(
-                [
-                    "Could not read contest from `",
-                    contest_path.to_str().unwrap(),
-                    "`",
-                ]
-                .concat(),
-            )),
+            contest_err: Some(ContestErr(format!(
+                "Could not read contest from {:?}",
+                contest_path,
+            ))),
             problem_errs: None,
         },
     }
@@ -161,7 +157,7 @@ impl Contest {
 
         for var in &self.template.1.vars {
             if !self.vars.contains_key(var) {
-                mapm_errs.push(ContestErr(["Does not contain key `", &var, "`"].concat()));
+                mapm_errs.push(ContestErr(format!("Does not contain key `{}`", &var)));
             }
         }
         if mapm_errs.len() > 0 {
@@ -176,43 +172,37 @@ impl Contest {
         let cwd = env::current_dir().unwrap();
         let mut results: Vec<Result<String, String>> = Vec::new();
 
-        let template_path = template_dir.join(&self.template.0);
+        let template_path = &template_dir.join(&self.template.0);
         let build_dir = Path::new("build");
         if !build_dir.is_dir() {
             // If `build` exists but is not a directory, user intervention is required - so we panic.
             if build_dir.exists() {
                 panic!(
-                    "`{}` exists in `{}` but is not a directory; please move it to fix this issue.",
-                    build_dir.to_str().unwrap(),
-                    cwd.to_str().unwrap()
+                    "{:?} exists in {:?} but is not a directory; please move it to fix this issue.",
+                    build_dir, cwd,
                 );
             }
             fs::create_dir(&build_dir).expect(&format!(
-                "Could not create directory `{}` in `{}`",
-                build_dir.to_str().unwrap(),
-                cwd.to_str().unwrap()
+                "Could not create directory {:?} in {:?}",
+                build_dir, cwd,
             ));
         }
 
         if !template_path.exists() {
             return (
-                Some(TemplateErr(
-                    [
-                        "Could not read template `",
-                        &template_path.to_str().unwrap(),
-                        "`",
-                    ]
-                    .concat(),
-                )),
+                Some(TemplateErr(format!(
+                    "Could not read template {:?}",
+                    template_path
+                ))),
                 results,
             );
         }
 
-        copy_dir_ignore_dots(&template_path, Path::new("build")).unwrap();
+        copy_dir_ignore_dots(template_path, Path::new("build")).unwrap();
 
         env::set_current_dir(&build_dir).expect(&format!(
-            "Could not set current directory to `{}`",
-            &build_dir.to_str().unwrap()
+            "Could not set current directory to {:?}",
+            &build_dir
         ));
 
         let mut headers = String::new();
@@ -234,14 +224,10 @@ impl Contest {
             self.problems.iter().map(|p| p.1.clone()).collect(),
         ));
 
-        fs::write("mapm-headers.tex", headers).expect(
-            &[
-                "Could not write `mapm-headers.tex` to directory `",
-                template_dir.to_str().unwrap(),
-                "`",
-            ]
-            .concat(),
-        );
+        fs::write("mapm-headers.tex", headers).expect(&format!(
+            "Could not write `mapm-headers.tex` to directory {:?}",
+            template_dir,
+        ));
 
         for (tex_path, unparsed_pdf_path) in &self.template.1.texfiles {
             let left_braces: Vec<usize> = unparsed_pdf_path
@@ -254,62 +240,31 @@ impl Contest {
                 .collect();
             if left_braces.len() != right_braces.len() {
                 return (
-                    Some(TemplateErr(
-                        [
-                            "Unbalanced braces for the value `",
-                            unparsed_pdf_path,
-                            "` of key `",
-                            tex_path,
-                            "` in the `texfiles` of template `",
-                            &self.template.0,
-                            "`",
-                        ]
-                        .concat(),
-                    )),
+                    Some(TemplateErr(format!(
+                        "Unbalanced braces for the value `{}` of key `{}` in the `texfiles` of template `{}`",
+                        unparsed_pdf_path,
+                        tex_path,
+                        &self.template.0,
+                    ))),
                     results,
                 );
             }
 
             let mut parsed_pdf_path = String::new();
 
+            // We reuse this errmsg, so define it here
+            let nested_braces_error = format!("Nested braces are not supported.\nSee the value `{}` of key `{}` in the `texfiles` of template `{}`", unparsed_pdf_path, tex_path, &self.template.0);
+
             if left_braces.len() > 0 {
                 for n in 0..left_braces.len() {
                     if left_braces[n] > right_braces[n] {
-                        return (
-                            Some(TemplateErr(
-                                [
-                                    "Nested braces are not supported.\nSee the value `",
-                                    unparsed_pdf_path,
-                                    "` of key `",
-                                    tex_path,
-                                    "` in the `texfiles` of template `",
-                                    &self.template.0,
-                                    "`",
-                                ]
-                                .concat(),
-                            )),
-                            results,
-                        );
+                        return (Some(TemplateErr(nested_braces_error)), results);
                     }
                 }
 
                 for n in 0..left_braces.len() - 1 {
                     if right_braces[n] > left_braces[n + 1] {
-                        return (
-                            Some(TemplateErr(
-                                [
-                                    "Nested braces are not supported.\nSee the value `",
-                                    unparsed_pdf_path,
-                                    "` of key `",
-                                    tex_path,
-                                    "` in the `texfiles` of template `",
-                                    &self.template.0,
-                                    "`",
-                                ]
-                                .concat(),
-                            )),
-                            results,
-                        );
+                        return (Some(TemplateErr(nested_braces_error)), results);
                     }
                 }
 
@@ -355,22 +310,21 @@ impl Contest {
             }
             let latexmk = Command::new("latexmk").args([&format!("-pdflatex={}", &self.template.1.engine), "-pdf", "-f", tex_path]).output().expect("Failed to execute latexmk.\nMake sure you have the latexmk script installed\nand make sure it is in your PATH.");
             if latexmk.status.success() {
-                results.push(Ok(["Finished compiling ", &parsed_pdf_path].concat()));
+                results.push(Ok(format!("Finished compiling `{}`", &parsed_pdf_path)));
                 let original_pdf_path = Path::new(tex_path).with_extension("pdf");
 
                 fs::rename(&original_pdf_path, &cwd.join(&parsed_pdf_path)).expect(&format!(
-                    "Could not rename `{}` to `{}`",
-                    &original_pdf_path.to_str().unwrap(),
-                    &parsed_pdf_path
+                    "Could not rename {:?} to `{}`",
+                    &original_pdf_path, &parsed_pdf_path
                 ));
             } else {
-                results.push(Err(["Failed to compile ", &parsed_pdf_path].concat()));
+                results.push(Err(format!("Failed to compile {}", &parsed_pdf_path)));
             }
         }
 
         env::set_current_dir(&cwd).expect(&format!(
-            "Failed to set current working directory to `{}`",
-            cwd.to_str().unwrap(),
+            "Failed to set current working directory to {:?}",
+            cwd
         ));
 
         (None, results)
